@@ -24,7 +24,9 @@ import datetime
 import hashlib
 import logging
 import os
+import simplejson
 import traceback
+import contextlib
 
 from django.contrib.auth.models import User
 from django.contrib.contenttypes import generic
@@ -42,6 +44,11 @@ from dashboard_app.helpers import BundleDeserializer
 from dashboard_app.managers import BundleManager
 from dashboard_app.repositories import RepositoryItem 
 from dashboard_app.repositories.data_report import DataReportRepository
+from dashboard_app.repositories.data_view import DataViewRepository
+
+# Fix some django issues we ran into
+from dashboard_app.patches import patch
+patch()
 
 
 def _help_max_length(max_length):
@@ -77,6 +84,22 @@ class SoftwarePackage(models.Model):
     def link_to_packages_ubuntu_com(self):
         return u"http://packages.ubuntu.com/{name}".format(name=self.name)
 
+
+class SoftwarePackageScratch(models.Model):
+    """
+    Staging area for SoftwarePackage data.
+
+    The code that keeps SoftwarePackage dumps data into here before more
+    carefully inserting it into the real SoftwarePackage table.
+
+    No data should ever be committed in this table.  It would be a temporary
+    table, but oddities in how the sqlite DB-API wrapper handles transactions
+    makes this impossible.
+    """
+    name = models.CharField(max_length=64)
+    version = models.CharField(max_length=64)
+
+
 class NamedAttribute(models.Model):
     """
     Model for adding generic named attributes
@@ -184,7 +207,7 @@ class BundleStream(RestrictedResource):
 
     @models.permalink
     def get_absolute_url(self):
-        return ("dashboard_app.test_run_list", [self.pathname])
+        return ("dashboard_app.views.bundle_list", [self.pathname])
 
     def get_test_run_count(self):
         return TestRun.objects.filter(bundle__bundle_stream=self).count()
@@ -341,7 +364,10 @@ class Bundle(models.Model):
 
     @models.permalink
     def get_absolute_url(self):
-        return ("dashboard_app.bundle.detail", [self.pk])
+        return ("dashboard_app.views.bundle_detail", [self.bundle_stream.pathname, self.content_sha1])
+
+    def get_permalink(self):
+        return reverse("dashboard_app.views.redirect_to_bundle", args=[self.content_sha1])
 
     def save(self, *args, **kwargs):
         if self.content:
@@ -413,6 +439,41 @@ class Bundle(models.Model):
             for attachment in test_run.attachments.all():
                 attachment.content.delete(save=save)
 
+    def get_sanitized_bundle(self):
+        self.content.open()
+        try:
+            return SanitizedBundle(self.content)
+        finally:
+            self.content.close()
+
+
+class SanitizedBundle(object):
+
+    def __init__(self, stream):
+        try:
+            self.bundle_json = simplejson.load(stream)
+            self.deserialization_error = None
+        except simplejson.JSONDeserializationError as ex:
+            self.bundle_json = None
+            self.deserialization_error = ex
+        self.did_remove_attachments = False
+        self._sanitize()
+
+    def get_human_readable_json(self):
+        return simplejson.dumps(self.bundle_json, indent=4)
+
+    def _sanitize(self):
+        for test_run in self.bundle_json.get("test_runs", []):
+            attachments = test_run.get("attachments")
+            if isinstance(attachments, list):
+                for attachment in attachments:
+                    attachment["content"] = None
+                    self.did_remove_attachments = True
+            elif isinstance(attachments, dict):
+                for name in attachments:
+                    attachments[name] = None
+                    self.did_remove_attachments = True
+
 
 class BundleDeserializationError(models.Model):
     """
@@ -466,7 +527,18 @@ class Test(models.Model):
 
     @models.permalink
     def get_absolute_url(self):
-        return ('dashboard_app.test.detail', [self.test_id])
+        return ('dashboard_app.views.test_detail', [self.test_id])
+
+    def count_results_without_test_case(self):
+        return TestResult.objects.filter(
+            test_run__test=self,
+            test_case=None).count()
+
+    def count_failures_without_test_case(self):
+        return TestResult.objects.filter(
+            test_run__test=self,
+            test_case=None,
+            result=TestResult.RESULT_FAIL).count()
 
 
 class TestCase(models.Model):
@@ -511,6 +583,9 @@ class TestCase(models.Model):
     def get_absolute_url(self):
         return ("dashboard_app.test_case.details", [self.test.test_id, self.test_case_id])
 
+    def count_failures(self):
+        return self.test_results.filter(result=TestResult.RESULT_FAIL).count()
+
 
 class SoftwareSource(models.Model):
     """
@@ -675,17 +750,28 @@ class TestRun(models.Model):
     @models.permalink
     def get_absolute_url(self):
         return ("dashboard_app.views.test_run_detail",
-                [self.analyzer_assigned_uuid])
+                [self.bundle.bundle_stream.pathname,
+                 self.bundle.content_sha1,
+                 self.analyzer_assigned_uuid])
 
-    def get_summary_results(self):
+    def get_permalink(self):
+        return reverse("dashboard_app.views.redirect_to_test_run", args=[self.analyzer_assigned_uuid])
+
+    def _get_summary_results(self, factor=3):
         stats = self.test_results.values('result').annotate(
             count=models.Count('result')).order_by()
         result = dict([
             (TestResult.RESULT_MAP[item['result']], item['count'])
             for item in stats])
         result['total'] = sum(result.values())
+        result['total_multiplied'] = result['total'] * factor
         return result
 
+    def get_summary_results(self):
+        if not hasattr(self, '_cached_summary_results'):
+            self._cached_summary_results = self._get_summary_results()
+        return self._cached_summary_results
+
     class Meta:
         ordering = ['-import_assigned_date']
 
@@ -723,10 +809,6 @@ class Attachment(models.Model):
     def __unicode__(self):
         return self.content_filename
 
-    @models.permalink
-    def get_absolute_url(self):
-        return ("dashboard_app.views.attachment_detail", [self.pk])
-
     def get_content_if_possible(self, mirror=False):
         if self.content:
             self.content.open()
@@ -752,6 +834,25 @@ class Attachment(models.Model):
             data = None
         return data
 
+    def is_test_run_attachment(self):
+        if (self.content_type.app_label == 'dashboard_app' and
+            self.content_type.model == 'testrun'):
+            return True
+
+    @property
+    def test_run(self):
+        if self.is_test_run_attachment():
+            return self.content_object
+
+    @models.permalink
+    def get_absolute_url(self):
+        if self.is_test_run_attachment():
+            return ("dashboard_app.views.attachment_detail",
+                    [self.test_run.bundle.bundle_stream.pathname,
+                     self.test_run.bundle.content_sha1,
+                     self.test_run.analyzer_assigned_uuid,
+                     self.pk])
+
 
 class TestResult(models.Model):
     """
@@ -845,6 +946,20 @@ class TestResult(models.Model):
     def __unicode__(self):
         return "Result {0}/{1}".format(self.test_run.analyzer_assigned_uuid, self.relative_index)
 
+    @models.permalink
+    def get_absolute_url(self):
+        return ("dashboard_app.views.test_result_detail", [
+            self.test_run.bundle.bundle_stream.pathname,
+            self.test_run.bundle.content_sha1,
+            self.test_run.analyzer_assigned_uuid,
+            self.relative_index,
+        ])
+
+    def get_permalink(self):
+        return reverse("dashboard_app.views.redirect_to_test_result",
+                       args=[self.test_run.analyzer_assigned_uuid,
+                             self.relative_index])
+
     @property
     def result_code(self):
         """
@@ -886,11 +1001,6 @@ class TestResult(models.Model):
 
     duration = property(_get_duration, _set_duration)
 
-    @models.permalink
-    def get_absolute_url(self):
-        return ("dashboard_app.views.test_result_detail",
-                [self.pk])
-
     def related_attachment_available(self):
         """
         Check if there is a log file attached to the test run that has
@@ -910,6 +1020,105 @@ class TestResult(models.Model):
         order_with_respect_to = 'test_run'
 
 
+class DataView(RepositoryItem):
+    """
+    Data view, a container for SQL query and optional arguments
+    """
+
+    repository = DataViewRepository()
+    
+    def __init__(self, name, backend_queries, arguments, documentation, summary):
+        self.name = name
+        self.backend_queries = backend_queries
+        self.arguments = arguments
+        self.documentation = documentation
+        self.summary = summary
+
+    def __unicode__(self):
+        return self.name
+
+    def __repr__(self):
+        return "<DataView name=%r>" % (self.name,)
+
+    @models.permalink
+    def get_absolute_url(self):
+        return ("dashboard_app.views.data_view_detail", [self.name])
+
+    def _get_connection_backend_name(self, connection):
+        backend = str(type(connection))
+        if "sqlite" in backend:
+            return "sqlite"
+        elif "postgresql" in backend:
+            return "postgresql"
+        else:
+            return ""
+
+    def get_backend_specific_query(self, connection):
+        """
+        Return BackendSpecificQuery for the specified connection
+        """
+        sql_backend_name = self._get_connection_backend_name(connection)
+        try:
+            return self.backend_queries[sql_backend_name]
+        except KeyError:
+            return self.backend_queries.get(None, None)
+
+    def lookup_argument(self, name):
+        """
+        Return Argument with the specified name
+
+        Raises LookupError if the argument cannot be found
+        """
+        for argument in self.arguments:
+            if argument.name == name:
+                return argument
+        raise LookupError(name)
+
+    @classmethod
+    def get_connection(cls):
+        """
+        Get the appropriate connection for data views
+        """
+        from django.db import connection, connections
+        from django.db.utils import ConnectionDoesNotExist
+        try:
+            return connections['dataview']
+        except ConnectionDoesNotExist:
+            logging.warning("dataview-specific database connection not available, dataview query is NOT sandboxed")
+            return connection  # NOTE: it's connection not connectionS (the default connection)
+
+    def __call__(self, connection, **arguments):
+        # Check if arguments have any bogus names
+        valid_arg_names = frozenset([argument.name for argument in self.arguments])
+        for arg_name in arguments:
+            if arg_name not in valid_arg_names:
+                raise TypeError("Data view %s has no argument %r" % (self.name, arg_name))
+        # Get the SQL template for our database connection
+        query = self.get_backend_specific_query(connection)
+        if query is None:
+            raise LookupError("Specified data view has no SQL implementation "
+                              "for current database")
+        # Replace SQL aruments with django placeholders (connection agnostic)
+        template = query.sql_template
+        template = template.replace("%", "%%")
+        # template = template.replace("{", "{{").replace("}", "}}")
+        sql = template.format(
+            **dict([
+                (arg_name, "%s")
+                for arg_name in query.argument_list]))
+        # Construct argument list using defaults for missing values
+        sql_args = [
+            arguments.get(arg_name, self.lookup_argument(arg_name).default)
+            for arg_name in query.argument_list]
+        with contextlib.closing(connection.cursor()) as cursor:
+            # Execute the query with the specified arguments
+            cursor.execute(sql, sql_args)
+            # Get and return the results
+            rows = cursor.fetchall()
+            columns = cursor.description
+            return rows, columns
+
+
 class DataReport(RepositoryItem):
     """
     Data reports are small snippets of xml that define
@@ -920,7 +1129,17 @@ class DataReport(RepositoryItem):
 
     def __init__(self, **kwargs):
         self._html = None
-        self.__dict__.update(kwargs)
+        self._data = kwargs
+
+    def __unicode__(self):
+        return self.title
+
+    def __repr__(self):
+        return "<DataReport name=%r>" % (self.name,)
+
+    @models.permalink
+    def get_absolute_url(self):
+        return ("dashboard_app.views.report_detail", [self.name])
 
     def _get_raw_html(self):
         pathname = os.path.join(self.base_path, self.path)
@@ -935,7 +1154,11 @@ class DataReport(RepositoryItem):
         return Template(self._get_raw_html())
 
     def _get_html_template_context(self):
-        return Context({"API_URL": reverse("dashboard_app.views.dashboard_xml_rpc_handler")})
+        from django.conf import settings
+        return Context({
+            "API_URL": reverse("dashboard_app.views.dashboard_xml_rpc_handler"),
+            "STATIC_URL": settings.STATIC_URL
+        })
 
     def get_html(self):
         from django.conf import settings
@@ -946,9 +1169,136 @@ class DataReport(RepositoryItem):
             self._html = template.render(context)
         return self._html
 
-    def __unicode__(self):
-        return self.title
+    @property
+    def title(self):
+        return self._data['title']
+
+    @property
+    def path(self):
+        return self._data['path']
+
+    @property
+    def name(self):
+        return self._data['name']
+
+    @property
+    def bug_report_url(self):
+        return self._data.get('bug_report_url')
+
+    @property
+    def author(self):
+        return self._data.get('author')
+
+
+class ImageHealth(object):
+
+    def __init__(self, rootfs_type, hwpack_type):
+        self.rootfs_type = rootfs_type
+        self.hwpack_type = hwpack_type
 
     @models.permalink
     def get_absolute_url(self):
-        return ("dashboard_app.views.report_detail", [self.name])
+        return ("dashboard_app.views.image_status_detail", [
+            self.rootfs_type, self.hwpack_type])
+
+    def get_tests(self):
+        return Test.objects.filter(test_runs__in=self.get_test_runs()).distinct()
+
+    def current_health_for_test(self, test):
+        test_run = self.get_current_test_run(test)
+        test_results = test_run.test_results
+        fail_count = test_results.filter(
+            result=TestResult.RESULT_FAIL).count()
+        pass_count = test_results.filter(
+            result=TestResult.RESULT_PASS).count()
+        total_count = test_results.count()
+        return {
+            "test_run": test_run,
+            "total_count": total_count,
+            "fail_count": fail_count,
+            "pass_count": pass_count,
+            "other_count": total_count - (fail_count + pass_count),
+            "fail_percent": fail_count * 100.0 / total_count if total_count > 0 else None,
+        }
+    
+    def overall_health_for_test(self, test):
+        test_run_list = self.get_all_test_runs_for_test(test)
+        test_result_list = TestResult.objects.filter(test_run__in=test_run_list)
+        fail_result_list = test_result_list.filter(result=TestResult.RESULT_FAIL)
+        pass_result_list = test_result_list.filter(result=TestResult.RESULT_PASS)
+
+        total_count = test_result_list.count()
+        fail_count = fail_result_list.count()
+        pass_count = pass_result_list.count()
+        fail_percent = fail_count * 100.0 / total_count if total_count > 0 else None
+        return {
+            "total_count": total_count,
+            "total_run_count": test_run_list.count(),
+            "fail_count": fail_count,
+            "pass_count": pass_count,
+            "other_count": total_count - fail_count - pass_count,
+            "fail_percent": fail_percent,
+        }
+
+    def get_recent_test_runs(self, last_n_days=7 * 2):
+        until = datetime.datetime.now()
+        since = until - datetime.timedelta(days=last_n_days)
+        return self.get_test_runs(
+        ).select_related(
+        ).filter(
+            analyzer_assigned_date__range=(since, until)
+        ).order_by('-analyzer_assigned_date')
+
+    def get_chart_data(self):
+        return TestResult.objects.select_related(
+        ).filter(
+            test_run__in=[run.pk for run in self.get_recent_test_runs().only('id').order_by()]
+        ).values(
+            'test_run__analyzer_assigned_date'
+        ).extra(
+            select={'bucket': 'date(analyzer_assigned_date)'},
+        ).values(
+            'bucket', 'result'
+        ).annotate(
+            count=models.Count('result')
+        ).order_by('result')
+
+    def get_test_runs(self):
+        return TestRun.objects.select_related(
+        ).filter(
+            bundle__bundle_stream__pathname="/anonymous/lava-daily/"
+        ).filter(
+            attributes__name='rootfs.type'
+        ).filter(
+            attributes__value=self.rootfs_type
+        ).filter(
+            attributes__name='hwpack.type'
+        ).filter(
+            attributes__value=self.hwpack_type
+        )
+
+    def get_current_test_run(self, test):
+        return self.get_all_test_runs_for_test(test).order_by('-analyzer_assigned_date')[0]
+
+    def get_all_test_runs_for_test(self, test):
+        return self.get_test_runs().filter(test=test)
+
+    @classmethod
+    def get_rootfs_list(self):
+        rootfs_list = [
+            attr['value'] 
+            for attr in NamedAttribute.objects.filter(
+                name='rootfs.type').values('value').distinct()]
+        try:
+            rootfs_list.remove('android')
+        except ValueError:
+            pass
+        return rootfs_list
+
+    @classmethod
+    def get_hwpack_list(self):
+        hwpack_list = [
+            attr['value'] 
+            for attr in NamedAttribute.objects.filter(
+                name='hwpack.type').values('value').distinct()]
+        return hwpack_list
